@@ -6,7 +6,7 @@
  * Descr.: boolean and Meta-constraint predicate management                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_bool.pl,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: fd_bool.pl,v 1.20 2012/03/15 09:39:42 diaz Exp $ */
 
 :-	built_in_fd.
 
@@ -90,6 +90,15 @@ LE #\\/ RE :-
 
 
 
+	% Reified interval
+
+fd_reified_in(X, L, U, B) :-
+	set_bip_name(fd_reified_in, 4),
+	'$call_c_test'('Pl_Fd_Reified_In'(X, L, U, B)).
+
+
+
+
 
         % Symbolic boolean constraints
 
