@@ -6,7 +6,7 @@
  * Descr.: arithmetic (inline) management - defs for meta-call             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arith_inl.pl,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: arith_inl.pl,v 1.19 2012/04/30 16:11:48 diaz Exp $ */
 
 :-	built_in.
 
@@ -66,3 +66,9 @@ X >= Y :-
 
 '$arith_eval'(X, Y) :-
 	'$call_c'('Pl_Arith_Eval_2'(X, Y)).
+
+
+succ(X, Y) :-
+	set_bip_name(succ, 2),
+	'$call_c_test'('Pl_Succ_2'(X, Y)).
+
