@@ -6,7 +6,7 @@
  * Descr.: byte-code support                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.c,v 1.23 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: bc_supp.c,v 1.26 2012/05/31 08:14:17 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -130,8 +130,14 @@ typedef enum
   PROCEED,
   FAIL,
 
+  GET_CURRENT_CHOICE_X,
+  GET_CURRENT_CHOICE_Y,
+
   CUT_X,
-  CUT_Y
+  CUT_Y,
+
+  SOFT_CUT_X,
+  SOFT_CUT_Y
 }
 BCCodOp;
 
@@ -314,7 +320,9 @@ Byte_Code_Initializer(void)
   Op_In_Tbl("proceed", PROCEED);
   Op_In_Tbl("fail", FAIL);
 
+  Op_In_Tbl("get_current_choice", GET_CURRENT_CHOICE_X);
   Op_In_Tbl("cut", CUT_X);
+  Op_In_Tbl("soft_cut", SOFT_CUT_X);
 
   nb_op = p - op_tbl;
 
@@ -623,7 +631,9 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
     case UNIFY_X_VARIABLE:
     case UNIFY_X_VALUE:
     case UNIFY_X_LOCAL_VALUE:
+    case GET_CURRENT_CHOICE_X:
     case CUT_X:
+    case SOFT_CUT_X:
       BC2_XY(w) = BC_Arg_X_Or_Y(*arg_adr, &op);
       break;
 
@@ -877,12 +887,12 @@ start:
     goto fail;
 
   arity = dyn->arity;
-  Pl_Load_Cut_Level(&A(arity));	/* init cut register */
+  A(arity) = Pl_Get_Current_Choice();	/* init cut register */
   A(arity + 1) = debug_call;
 
   clause = Pl_Scan_Dynamic_Pred(func, arity, dyn, A(0),
-			     (PlLong (*)()) BC_Emulate_Pred_Alt,
-			     DYN_ALT_FCT_FOR_JUMP, arity + 2, &A(0));
+				(PlLong (*)()) BC_Emulate_Pred_Alt,
+				DYN_ALT_FCT_FOR_JUMP, arity + 2, &A(0));
   if (clause == NULL)
     goto fail;
 
@@ -1375,6 +1385,16 @@ bc_loop:
 	}
       goto fail;
 
+    case GET_CURRENT_CHOICE_X:
+      x = BC2_XY(w);
+      X(x) = Pl_Get_Current_Choice();
+      goto bc_loop;
+
+    case GET_CURRENT_CHOICE_Y:
+      y = BC2_XY(w);
+      Y(E, y) = Pl_Get_Current_Choice();
+      goto bc_loop;
+
     case CUT_X:
       x = BC2_XY(w);
       Pl_Cut(X(x));
@@ -1384,6 +1404,16 @@ bc_loop:
       y = BC2_XY(w);
       Pl_Cut(Y(E, y));
       goto bc_loop;
+
+    case SOFT_CUT_X:
+      x = BC2_XY(w);
+      Pl_Soft_Cut(X(x));
+      goto bc_loop;
+
+    case SOFT_CUT_Y:
+      y = BC2_XY(w);
+      Pl_Soft_Cut(Y(E, y));
+      goto bc_loop;
     }
 
 fail:
