@@ -6,7 +6,7 @@
  * Descr.: debugger - C part                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,11 +35,10 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: debugger_c.c,v 1.22 2011/04/08 16:16:16 diaz Exp $ */
+/* $Id: debugger_c.c,v 1.26 2012/06/20 13:07:35 diaz Exp $ */
 
 #include <string.h>
 #include <stdlib.h>
-#include <signal.h>
 #include <setjmp.h>
 
 #if defined(_WIN32) || defined(__CYGWIN__)
@@ -108,7 +107,9 @@ static WamWord reg_copy[NB_OF_REGS];
 static StmInf *pstm_i;
 static StmInf *pstm_o;
 
-static jmp_buf dbg_jumper;
+static sigjmp_buf dbg_jumper;
+
+static void *invalid_addr;
 
 
 
@@ -337,35 +338,20 @@ Pl_Choice_Point_Arg_3(WamWord b_word, WamWord i_word, WamWord arg_word)
 
 
 
-#if defined(USE_SEH)
-
 /*-------------------------------------------------------------------------*
- * DEBUGGER_SEH_HANDLER                                                    *
+ * DEBUGGER_SIGSEGV_HANDLER                                                *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static EXCEPT_DISPOSITION
-Debugger_SEH_Handler(EXCEPTION_RECORD *excp_rec, void *establisher_frame,
-		     CONTEXT *context_rec, void *dispatcher_cxt)
+static int
+Debugger_SIGSEGV_Handler(void *bad_addr)
 {
-  if (excp_rec->ExceptionFlags)
-    return ExceptContinueSearch; /* unwind and others */
+  invalid_addr = bad_addr;
 
-  longjmp(dbg_jumper, excp_rec->ExceptionCode);
+  siglongjmp(dbg_jumper, 1);
+
+  return 1;
 }
 
-#else
-
-/*-------------------------------------------------------------------------*
- * DEBUGGER_SIGNAL_HANDLER                                                 *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static void
-Debugger_Signal_Handler(int sig)
-{
-  longjmp(dbg_jumper, sig);
-}
-
-#endif
 
 
 
@@ -386,26 +372,20 @@ Pl_Debug_Wam(void)
 
   Pl_Stream_Printf(pstm_o, "Welcome to the WAM debugger - experts only\n");
 
-#if defined(USE_SEH)
-  SEH_PUSH(Debugger_SEH_Handler);
-#endif
+  Pl_Push_SIGSEGV_Handler(Debugger_SIGSEGV_Handler);
 
-  ret = setjmp(dbg_jumper);
-  if (ret == 0)
+ restart:
+
+  ret = sigsetjmp(dbg_jumper, 1);
+  if (ret != 0)
     {
-#if !defined(USE_SEH)
-      signal(SIGSEGV, Debugger_Signal_Handler);
-#endif
-    }
-  else
-    {
-      Pl_Stream_Printf(pstm_o, "ERROR from handler: %d (%#x)\n", ret, ret);
+      Pl_Stream_Printf(pstm_o, "SIGSEGV occured at: %p\n", invalid_addr);
+      goto restart;
     }
 
   for (;;)
     {
-      if (Pl_Stream_Gets_Prompt(prompt, pstm_o,
-			     str, sizeof(str), pstm_i) == NULL)
+      if (Pl_Stream_Gets_Prompt(prompt, pstm_o, str, sizeof(str), pstm_i) == NULL)
 	break;
 
       Scan_Command(str);
@@ -416,9 +396,8 @@ Pl_Debug_Wam(void)
       if (command)
 	(*command) ();
     }
-#if defined(USE_SEH)
-  SEH_POP;
-#endif
+
+  Pl_Pop_SIGSEGV_Handler();
 }
 
 
@@ -526,8 +505,8 @@ Write_Data_Modify(void)
 
 	  if (*read_arg[0] == 'w')
 	    Pl_Write_Term(pstm_o, -1,
-		       MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
-		       adr[offset]);
+			  MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL,
+			  adr[offset]);
 	  else
 	    {
 	      Print_Wam_Word(adr + offset);
@@ -754,7 +733,7 @@ Backtrack(void)
 	    {
 	      Pl_Stream_Printf(pstm_o, " for ");
 	      Pl_Write_Term(pstm_o, -1, MAX_PREC,
-			 WRITE_NUMBER_VARS | WRITE_NAME_VARS, AB(adr, 0));
+			    WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL, AB(adr, 0));
 	    }
 
 	  Pl_Stream_Printf(pstm_o, "\n");
@@ -787,8 +766,8 @@ Backtrack(void)
   if (arity == 0 && strcmp(pl_atom_tbl[func].name, "$clause_alt") == 0)
     {
       Pl_Stream_Printf(pstm_o, " for ");
-      Pl_Write_Term(pstm_o, -1, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
-		 AB(adr, 0));
+      Pl_Write_Term(pstm_o, -1, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL,
+		    AB(adr, 0));
     }
   Pl_Stream_Printf(pstm_o, "\n");
 
