@@ -6,7 +6,7 @@
  * Descr.: Prolog errors support                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: error_supp.c,v 1.27 2011/05/27 16:13:52 diaz Exp $ */
+/* $Id: error_supp.c,v 1.29 2012/01/10 17:46:01 diaz Exp $ */
 
 #include <errno.h>
 #include <string.h>
@@ -504,9 +504,9 @@ Pl_Unknown_Pred_Error(int func, int arity)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Os_Error(void)
+Pl_Os_Error(int ret_val)
 {
-  char *err_str = Pl_M_Sys_Err_String(errno);
+  char *err_str = Pl_M_Sys_Err_String(ret_val);
 
   if (Flag_Value(FLAG_OS_ERROR) == FLAG_VALUE_ERROR)
     Pl_Err_System(Pl_Create_Allocate_Atom(err_str));
@@ -514,8 +514,8 @@ Pl_Os_Error(void)
   Update_Cur_From_C_Bip();
   if (Flag_Value(FLAG_OS_ERROR) == FLAG_VALUE_WARNING)
     Pl_Stream_Printf(pl_stm_tbl[pl_stm_top_level_output],
-		  "warning: OS error: %s (from %s)\n",
-		  err_str, Context_Error_String());
+		     "warning: OS error: %s (from %s)\n",
+		     err_str, Context_Error_String());
 }
 
 
