@@ -6,7 +6,7 @@
  * Descr.: term expansion management                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: expand.pl,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: expand.pl,v 1.20 2012/04/30 16:11:48 diaz Exp $ */
 
 :-	built_in.
 
@@ -88,10 +88,11 @@ phrase(DcgBody, In, Out) :-
 	;
 	    true
 	),
-	'$check_list_or_partial_list'(In),
-	'$check_list_or_partial_list'(Out),
-	'$dcg_trans_body'(DcgBody, In, Out, Body),
-	'$call'(Body, phrase, Arity, true).
+	% '$check_list_or_partial_list'(In),
+	% '$check_list_or_partial_list'(Out),
+	'$dcg_trans_body'(DcgBody, In, Out1, Body),
+	'$call'(Body, phrase, Arity, true),
+	Out = Out1.
 
 
 
