@@ -6,7 +6,7 @@
  * Descr.: Prolog flag and system variable management - C Part             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag_c.c,v 1.30 2011/06/01 12:50:20 diaz Exp $ */
+/* $Id: flag_c.c,v 1.34 2012/05/31 08:14:17 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -796,7 +796,7 @@ Pl_Get_Current_B_1(WamWord b_word)
 {
   WamWord word;
 
-  Pl_Load_Cut_Level(&word);
+  word = Pl_Get_Current_Choice();
   Pl_Unify(word, b_word);
 }
 
@@ -825,8 +825,8 @@ Pl_Set_Current_B_1(WamWord b_word)
  *-------------------------------------------------------------------------*/
 
 /* these macros are to avoid gcc warning warn_unused_result */
-#define FWRITE(b, sz, n, f) if (fwrite(b, sz, n, f) != n) ;
-#define FREAD(b, sz, n, f)  if (fread(b, sz, n, f) != n) ;
+#define FWRITE(b, sz, n, f) if (fwrite(b, sz, n, f) != n) {}
+#define FREAD(b, sz, n, f)  if (fread(b, sz, n, f) != n) {}
 
 
 Bool
@@ -848,7 +848,7 @@ Pl_Write_Pl_State_File(WamWord file_word)
   file = Pl_M_Absolute_Path_Name(file);
 
   f = fopen(file, "wb");
-  Os_Test_Error(f == NULL);
+  Os_Test_Error_Null(f);
 
   i = Pl_Hash_Nb_Elements(pl_oper_tbl);
   FWRITE(&i, sizeof(i), 1, f);
@@ -927,7 +927,7 @@ Pl_Read_Pl_State_File(WamWord file_word)
   file = Pl_M_Absolute_Path_Name(file);
 
   f = fopen(file, "rb");
-  Os_Test_Error(f == NULL);
+  Os_Test_Error_Null(f);
 
   Pl_Hash_Delete_All(pl_oper_tbl);
 
