@@ -6,7 +6,7 @@
  * Descr.: global variable (inline) management - C part                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: g_var_inl_c.c,v 1.21 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: g_var_inl_c.c,v 1.23 2012/01/11 11:52:39 diaz Exp $ */
 
 #include <stdlib.h>
 
@@ -843,7 +843,7 @@ Get_Target_From_Gvar(WamWord gvar_word)
   WamWord word1;
   int atom;
   int arity;
-  WamWord *arg_adr, *g_arg;
+  WamWord *arg_adr;
   GVarElt *g_elem, *g_end;
   GVarElt *p;
   int i, j, size;
@@ -851,9 +851,6 @@ Get_Target_From_Gvar(WamWord gvar_word)
   PlLong index;
   GTarget *gt = &g_target;
 
-
-  g_arg = NULL;
-
   arg_adr = Pl_Rd_Callable_Check(gvar_word, &atom, &arity);
 
   if (atom == ATOM_CHAR('-') && arity == 2)
