@@ -6,7 +6,7 @@
  * Descr.: parser support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: parse_supp.c,v 1.24 2011/05/16 13:36:07 diaz Exp $ */
+/* $Id: parse_supp.c,v 1.27 2012/06/20 13:07:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -80,7 +80,7 @@ static Bool tok_present;
 static TokInf unget_tok;
 
 
-static jmp_buf jumper;
+static sigjmp_buf jumper;
 
 #if !defined(NO_USE_REGS) && NB_OF_USED_MACHINE_REGS > 0
 static WamWord buff_save_machine_regs[NB_OF_USED_MACHINE_REGS];
@@ -217,7 +217,7 @@ Pl_Read_Term(StmInf *pstm, int parse_end_of_term)
   tok_present = FALSE;
   Save_Machine_Regs(buff_save_machine_regs);
 
-  jmp_val = setjmp(jumper);
+  jmp_val = sigsetjmp(jumper, 1);
 
   Restore_Machine_Regs(buff_save_machine_regs);
 
@@ -709,9 +709,7 @@ Lookup_In_Dico_Var(char *name)
   if (name[0] != '_' || name[1] != '\0')
     {
       named = TRUE;
-      for (i = 0;
-	   i < pl_parse_nb_var && strcmp(name, pl_parse_dico_var[i].name) != 0;
-	   i++)
+      for (i = 0; i < pl_parse_nb_var && strcmp(name, pl_parse_dico_var[i].name) != 0; i++)
 	;
     }
   else
@@ -752,7 +750,7 @@ Parse_Error(char *err_msg)
     Pl_Recover_After_Error(pstm_i);
 
   Save_Machine_Regs(buff_save_machine_regs);
-  longjmp(jumper, 1);
+  siglongjmp(jumper, 1);
 }
 
 
