@@ -6,7 +6,7 @@
  * Descr.: pretty print clause management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pretty_c.c,v 1.22 2011/05/27 16:13:52 diaz Exp $ */
+/* $Id: pretty_c.c,v 1.28 2012/05/31 08:14:17 diaz Exp $ */
 
 #define OBJ_INIT Pretty_Initializer
 
@@ -74,6 +74,7 @@
 static int atom_clause;
 static int atom_dcg;
 static int atom_if;
+static int atom_soft_if;	/* soft-cut */
 
 static int atom_dollar_var;
 static int atom_dollar_varname;
@@ -87,6 +88,8 @@ static int nb_singl_var;
 
 static int nb_to_try;
 
+static WamWord *above_H;
+
 
 
 
@@ -128,6 +131,7 @@ Pretty_Initializer(void)
   atom_clause = Pl_Create_Atom(":-");
   atom_dcg = Pl_Create_Atom("-->");
   atom_if = Pl_Create_Atom("->");
+  atom_soft_if = Pl_Create_Atom("*->");
 
   atom_dollar_var = Pl_Create_Atom("$VAR");
   atom_dollar_varname = Pl_Create_Atom("$VARNAME");
@@ -141,14 +145,15 @@ Pretty_Initializer(void)
 
 
 /*-------------------------------------------------------------------------*
- * PL_PORTRAY_CLAUSE_2                                                     *
+ * PL_PORTRAY_CLAUSE_3                                                     *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Portray_Clause_2(WamWord sora_word, WamWord term_word)
+Pl_Portray_Clause_3(WamWord sora_word, WamWord term_word, WamWord above_word)
 {
   int stm;
   StmInf *pstm;
+  WamWord *b;
 
 
   stm = (sora_word == NOT_A_WAM_WORD)
@@ -158,6 +163,9 @@ Pl_Portray_Clause_2(WamWord sora_word, WamWord term_word)
   pl_last_output_sora = sora_word;
   Pl_Check_Stream_Type(stm, TRUE, FALSE);
 
+  b = LSSA + Pl_Rd_Integer(above_word); /* see Pl_Get_Current_Choice / Pl_Cut */
+  above_H = HB(b);
+
   Portray_Clause(pstm, term_word);
 }
 
@@ -165,13 +173,13 @@ Pl_Portray_Clause_2(WamWord sora_word, WamWord term_word)
 
 
 /*-------------------------------------------------------------------------*
- * PL_PORTRAY_CLAUSE_1                                                     *
+ * PL_PORTRAY_CLAUSE_2                                                     *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Portray_Clause_1(WamWord term_word)
+Pl_Portray_Clause_2(WamWord term_word, WamWord above_word)
 {
-  Pl_Portray_Clause_2(NOT_A_WAM_WORD, term_word);
+  Pl_Portray_Clause_3(NOT_A_WAM_WORD, term_word, above_word);
 }
 
 
@@ -194,7 +202,7 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 
   if (Check_Structure(term_word, atom_clause, 2, arg_word))
     {
-      Pl_Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, arg_word[0]);
+      Pl_Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, above_H, arg_word[0]);
       DEREF(arg_word[1], word, tag_mask);
       atom = UnTag_ATM(word);
       if (tag_mask != TAG_ATM_MASK || atom != pl_atom_true)
@@ -203,14 +211,13 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 	  Start_Line(pstm, 0, ' ');
 	  Show_Body(pstm, 0, GENERAL_BODY, arg_word[1]);
 	}
-      Pl_Write_A_Char(pstm, '.');
-      Pl_Write_A_Char(pstm, '\n');
+      Pl_Write_A_Full_Stop(pstm);
       return;
     }
 
   if (Check_Structure(term_word, atom_dcg, 2, arg_word))
     {
-      Pl_Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, arg_word[0]);
+      Pl_Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, above_H, arg_word[0]);
       DEREF(arg_word[1], word, tag_mask);
       atom = UnTag_ATM(word);
       if (tag_mask != TAG_ATM_MASK || atom != pl_atom_true)
@@ -219,8 +226,7 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 	  Start_Line(pstm, 0, ' ');
 	  Show_Body(pstm, 0, GENERAL_BODY, arg_word[1]);
 	}
-      Pl_Write_A_Char(pstm, '.');
-      Pl_Write_A_Char(pstm, '\n');
+      Pl_Write_A_Full_Stop(pstm);
       return;
     }
 
@@ -228,14 +234,12 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
     {
       Pl_Stream_Puts(":-\t", pstm);
       Show_Body(pstm, 0, GENERAL_BODY, arg_word[0]);
-      Pl_Write_A_Char(pstm, '.');
-      Pl_Write_A_Char(pstm, '\n');
+      Pl_Write_A_Full_Stop(pstm);
       return;
     }
 
-  Pl_Write_Term(pstm, -1, MAX_PREC, WRITE_MASK, term_word);
-  Pl_Write_A_Char(pstm, '.');
-  Pl_Write_A_Char(pstm, '\n');
+  Pl_Write_Term(pstm, -1, MAX_PREC, WRITE_MASK, above_H, term_word);
+  Pl_Write_A_Full_Stop(pstm);
 }
 
 
@@ -299,8 +303,8 @@ static void
 Show_Body(StmInf *pstm, int level, int context, WamWord body_word)
 {
   WamWord arg_word[2];
-  static
-    int prec[] = { 1200 - 1, 1000 - 1, 1000, 1100 - 1, 1100, 1050 - 1, 1050 };
+  int soft_cut;
+  static int prec[] = { 1200 - 1, 1000 - 1, 1000, 1100 - 1, 1100, 1050 - 1, 1050 };
 
 
   if (Check_Structure(body_word, ATOM_CHAR(','), 2, arg_word))
@@ -337,7 +341,9 @@ Show_Body(StmInf *pstm, int level, int context, WamWord body_word)
     }
 
 
-  if (Check_Structure(body_word, atom_if, 2, arg_word))
+  soft_cut = 0;
+  if (Check_Structure(body_word, atom_if, 2, arg_word) ||
+      (soft_cut = 1, Check_Structure(body_word, atom_soft_if, 2, arg_word)))
     {
       if (context != LEFT_OR && context != RIGHT_OR)
 	{
@@ -346,7 +352,7 @@ Show_Body(StmInf *pstm, int level, int context, WamWord body_word)
 	}
 
       Show_Body(pstm, level, LEFT_IF, arg_word[0]);
-      Pl_Stream_Puts(" ->", pstm);
+      Pl_Stream_Puts((soft_cut == 0) ? " ->" : " *-> ", pstm);
       Start_Line(pstm, level, ' ');
       Show_Body(pstm, level, RIGHT_IF, arg_word[1]);
 
@@ -359,7 +365,7 @@ Show_Body(StmInf *pstm, int level, int context, WamWord body_word)
       return;
     }
 
-  Pl_Write_Term(pstm, -1, prec[context], WRITE_MASK, body_word);
+  Pl_Write_Term(pstm, -1, prec[context], WRITE_MASK, above_H, body_word);
 }
 
 
@@ -408,9 +414,13 @@ Pl_Name_Singleton_Vars_1(WamWord start_word)
 {
   WamWord word;
 
+  if (!Pl_Acyclic_Term_1(start_word))
+    return;
+
   singl_var_ptr = pl_glob_dico_var;	/* pl_glob_dico_var: stores singletons */
   nb_singl_var = 0;
 
+
   Pl_Treat_Vars_Of_Term(start_word, FALSE, Collect_Singleton);
 
   if (nb_singl_var == 0)
@@ -563,7 +573,8 @@ Pl_Bind_Variables_4(WamWord term_word, WamWord exclude_list_word,
       lst_adr = UnTag_LST(word);
 
       DEREF(Car(lst_adr), word, tag_mask);
-      Collect_Excluded_Rec(word);
+      if (Pl_Acyclic_Term_1(word))
+	Collect_Excluded_Rec(word);
 
       stc_adr = UnTag_STC(word);
       if (tag_mask == TAG_STC_MASK && Functor_And_Arity(stc_adr) == equal_2)
@@ -576,7 +587,8 @@ Pl_Bind_Variables_4(WamWord term_word, WamWord exclude_list_word,
       exclude_list_word = Cdr(lst_adr);
     }
 
-  Pl_Treat_Vars_Of_Term(term_word, FALSE, Bind_Variable);
+  if (Pl_Acyclic_Term_1(term_word))
+    Pl_Treat_Vars_Of_Term(term_word, FALSE, Bind_Variable);
 
   return Pl_Un_Integer_Check(nb_to_try, next_word);
 }
