@@ -6,7 +6,7 @@
  * Descr.: stream selection and control management - C part                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_c.c,v 1.23 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: stream_c.c,v 1.26 2012/01/11 11:52:39 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -1049,11 +1049,10 @@ Pl_Current_Mirror_2(WamWord stm_word, WamWord m_stm_word)
 
   if (m->next != NULL) /* non deterministic case */
     {
-      A(0) = stm;
+      A(0) = stm;		/* useless in fact */
       A(1) = m_stm_word;
       A(2) = (WamWord) m->next;
-      Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_MIRROR_ALT, 0),
-			  3);
+      Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_MIRROR_ALT, 0), 3);
     }
 
   return Pl_Get_Integer(m->stm, m_stm_word);
@@ -1069,13 +1068,13 @@ Pl_Current_Mirror_2(WamWord stm_word, WamWord m_stm_word)
 Bool
 Pl_Current_Mirror_Alt_0(void)
 {
-  int stm;
+  /*  int stm; */
   WamWord m_stm_word;
   StmLst *m;
 
   Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_MIRROR_ALT, 0), 0);
 
-  stm = AB(B, 0);
+  /*  stm = AB(B, 0); */
   m_stm_word = AB(B, 1);
   m = (StmLst *) AB(B, 2);
 
