@@ -6,7 +6,7 @@
  * Descr.: term support                                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: term_supp.c,v 1.24 2011/06/10 16:44:02 diaz Exp $ */
+/* $Id: term_supp.c,v 1.27 2012/05/31 08:14:17 diaz Exp $ */
 
 #include <string.h>
 
@@ -656,6 +656,7 @@ Pl_Acyclic_Term_1(WamWord start_word)
 
 {
   WamWord word, tag_mask;
+  WamWord word1;
   WamWord *adr;
   int arity;
   Bool ret;
@@ -680,17 +681,23 @@ Pl_Acyclic_Term_1(WamWord start_word)
 
   while (--arity >= 0)
     {
-      word = *adr;
-      if (word == MARK)		/* marked = cyclic */
+      word1 = *adr;
+      DEREF(word1, word, tag_mask);
+
+      if (word == MARK)	/* marked = cyclic */
 	return FALSE;
 
-      *adr = MARK;		/* mark it */
+      if (tag_mask == TAG_LST_MASK || tag_mask == TAG_STC_MASK)
+	{
+	  *adr = MARK;		/* mark it */
 
-      ret = Pl_Acyclic_Term_1(word);
+	  ret = Pl_Acyclic_Term_1(word1);
       
-      *adr++ = word;		/* unmark it */
-      if (!ret)
-	return FALSE;
+	  *adr = word1;		/* unmark it */
+	  if (!ret)
+	    return FALSE;
+	}
+      adr++;
     }
 
   return TRUE;
