@@ -6,7 +6,7 @@
  * Descr.: top Level                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_level.pl,v 1.26 2011/05/27 16:13:52 diaz Exp $ */
+/* $Id: top_level.pl,v 1.29 2012/05/03 16:33:01 diaz Exp $ */
 
 :-	built_in.
 
@@ -106,6 +106,15 @@ break :-
 
 
 
+'$top_level_exception'('$post_query_exception'(X)) :-
+	'$reinit_after_exception',
+	!,
+	format(top_level_output, '~Ntop-level exception: ', []),
+	write_term(top_level_output, X, [quoted(true), numbervars(false), namevars(false)]),
+	nl(top_level_output),
+	fail.
+	
+
 '$top_level_exception'(X) :-
 	'$reinit_after_exception',
 	format(top_level_output, '~Nuncaught exception: ', []),
@@ -231,7 +240,8 @@ break :-
 	'$call_c'('Pl_Save_Regs_For_Signal'),  % save some registers in case of CTRL+C
 	'$get_current_B'(B1),
 	format(top_level_output, '~N', []),
-	'$set_query_vars_names'(QueryVars, ToDispVars),
+	'$catch_internal'('$set_query_vars_names'(QueryVars, ToDispVars), Err, throw('$post_query_exception'(Err)), false),
+%	'$set_query_vars_names'(QueryVars, ToDispVars),
 	(   fail,                             % do not activate 'alt if vars'
 	    ToDispVars = [] ->
 	    true                              % no alt if only anonymous vars
@@ -288,8 +298,12 @@ break :-
 '$write_solution1'([], _).
 
 '$write_solution1'([Name = Value|ToDispVars], Prior) :-
-	format(top_level_output, '~n~a = ', [Name]),
-	write_term(top_level_output, Value, [quoted(true), numbervars(false), namevars(true), priority(Prior)]),
+	(   acyclic_term(Value) ->
+	    format(top_level_output, '~n~a = ', [Name]),
+	    write_term(top_level_output, Value, [quoted(true), numbervars(false), namevars(true), priority(Prior)])
+	;
+	    format(top_level_output, '~ncannot display cyclic term for ~a', [Name])
+	),
 	'$write_solution1'(ToDispVars, Prior).
 
 
