@@ -6,7 +6,7 @@
  * Descr.: term output (write/1 and friends) management                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: write.pl,v 1.20 2011/05/11 15:11:11 diaz Exp $ */
+/* $Id: write.pl,v 1.23 2012/04/30 16:11:48 diaz Exp $ */
 
 :-	built_in.
 
@@ -98,11 +98,11 @@ display(SorA, Term) :-
 
           % option mask in sys_var[0]: (see write_supp.h)
           %
-          %    b5         b4        b3         b2           b1         b0
-          %    0/1        0/1       0/1        0/1          0/1        0/1
-          % portrayed  space_args  namevars  numbervars  ignore_ops  quoted
-          %  0=false    0=false    0=false    0=false      0=false   0=false
-          %  1=true     1=true     1=true     1=true       1=true    1=true
+          %    b6          b5         b4        b3         b2           b1         b0
+          %    0/1         0/1        0/1       0/1        0/1          0/1        0/1
+          % var_names   portrayed  space_args  namevars  numbervars  ignore_ops  quoted
+          %  0=false     0=false    0=false    0=false    0=false      0=false   0=false
+          %  1=true      1=true     1=true     1=true     1=true       1=true    1=true
           %
           % max_depth in sys_var[1]
           % priority in sys_var[2]
@@ -112,29 +112,41 @@ write_term(Term, Options) :-
 	set_bip_name(write_term, 2),
 	'$set_write_defaults',
 	'$get_write_options'(Options),
-	'$call_c'('Pl_Write_Term_1'(Term)).
+	'$call_c'('Pl_Write_Term_1'(Term)),
+	fail.
 
+write_term(_, _).
+	
 
 write_term(SorA, Term, Options) :-
 	set_bip_name(write_term, 3),
 	'$set_write_defaults',
 	'$get_write_options'(Options),
-	'$call_c'('Pl_Write_Term_2'(SorA, Term)).
+	'$call_c'('Pl_Write_Term_2'(SorA, Term)),
+	fail.
 
+write_term(_, _, _).
 
 
 
 '$set_write_defaults' :-
 	'$sys_var_write'(0, 0),                               % default mask
 	'$sys_var_write'(1, -1),
-	'$sys_var_write'(2, 1200).
+	'$sys_var_write'(2, 1200),
+	'$sys_var_write'(3, 0).
 
 
 
 
 '$get_write_options'(Options) :-
 	'$check_list'(Options),
-	'$get_write_options1'(Options).
+	'$get_write_options1'(Options),
+	(   '$sys_var_get_bit'(0, 6, 1) -> % variable_names ==> namevars
+	    '$sys_var_set_bit'(0, 3)
+	;
+	    true
+	).
+	    
 
 
 '$get_write_options1'([]).
@@ -180,6 +192,10 @@ write_term(SorA, Term, Options) :-
 	    '$sys_var_set_bit'(0, 3)
 	).
 
+'$get_write_options2'('$above'(X)) :- % "above" choice-point for numbervars/namevars
+	integer(X),
+	'$sys_var_write'(3, X).
+
 '$get_write_options2'(space_args(X)) :-
 	nonvar(X),
 	(   X = false,
@@ -196,6 +212,10 @@ write_term(SorA, Term, Options) :-
 	    '$sys_var_set_bit'(0, 5)
 	).
 
+'$get_write_options2'(variable_names(VarNames)) :-
+	'$sys_var_set_bit'(0, 6),
+	'$name_variables'(VarNames).
+
 '$get_write_options2'(max_depth(X)) :-
 	integer(X),
 	'$sys_var_write'(1, X).
@@ -209,6 +229,18 @@ write_term(SorA, Term, Options) :-
 
 
 
+'$name_variables'([]).
+
+'$name_variables'([Name = Var|VarNames]) :-
+	('$is_valid_var_name'(Name), Var = '$VARNAME'(Name), ! ; true),
+	'$name_variables'(VarNames).
+
+
+
+'$is_valid_var_name'(Name) :-
+	'$call_c_test'('Pl_Is_Valid_Var_Name_1'(Name)).
+
+
 
 nl :-
 	set_bip_name(nl, 0),
