@@ -6,7 +6,7 @@
  * Descr.: term output (write/1 and friends) management - C part           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: write_c.c,v 1.18 2011/05/18 12:46:49 diaz Exp $ */
+/* $Id: write_c.c,v 1.21 2012/05/31 08:14:17 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -71,6 +71,7 @@ Pl_Write_Term_2(WamWord sora_word, WamWord term_word)
 {
   int stm;
   StmInf *pstm;
+  WamWord *above_H = NULL;
 
 
   stm = (sora_word == NOT_A_WAM_WORD)
@@ -80,8 +81,15 @@ Pl_Write_Term_2(WamWord sora_word, WamWord term_word)
   pl_last_output_sora = sora_word;
   Pl_Check_Stream_Type(stm, TRUE, FALSE);
 
+  if (SYS_VAR_WRITE_ABOVE > 0)
+    {
+      WamWord *b = LSSA + SYS_VAR_WRITE_ABOVE; /* see Pl_Get_Current_Choice / Pl_Cut */
+      above_H = HB(b);
+    }
+
+
   Pl_Write_Term(pstm, SYS_VAR_WRITE_DEPTH, SYS_VAR_WRITE_PREC,
-		SYS_VAR_OPTION_MASK, term_word);
+		SYS_VAR_OPTION_MASK, above_H, term_word);
 }
 
 
