@@ -6,7 +6,7 @@
  * Descr.: write term support                                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: write_supp.c,v 1.28 2011/06/09 14:47:38 diaz Exp $ */
+/* $Id: write_supp.c,v 1.33 2012/04/30 16:11:48 diaz Exp $ */
 
 #include <string.h>
 #include <ctype.h>
@@ -109,6 +109,8 @@ static Bool name_vars;
 static Bool space_args;
 static Bool portrayed;
 
+static WamWord *name_number_above_H;
+
 static int last_prefix_op = W_NO_PREFIX_OP;
 static Bool *p_bracket_minus;
 
@@ -173,7 +175,8 @@ Write_Supp_Initializer(void)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord term_word)
+Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord *above_H, 
+	      WamWord term_word)
 {
   pstm_o = pstm;
 
@@ -184,6 +187,8 @@ Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord term_word)
   space_args = mask & WRITE_SPACE_ARGS;
   portrayed = mask & WRITE_PORTRAYED;
 
+  name_number_above_H = above_H;
+
   pl_last_writing = W_NOTHING;
 
   Show_Term(depth, prec, (prec >= 1200) ? GENERAL_TERM : INSIDE_ANY_OP, term_word);
@@ -201,8 +206,8 @@ Pl_Write_Simple(WamWord term_word)
 {
   StmInf *pstm = pl_stm_tbl[pl_stm_output];
 
-  Pl_Write_Term(pstm, -1, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
-	     term_word);
+  Pl_Write_Term(pstm, -1, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL,
+		term_word);
   /* like write/1 */
 }
 
@@ -312,6 +317,24 @@ Need_Space(int c)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_WRITE_A_FULL_STOP                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Write_A_Full_Stop(StmInf *pstm)
+{
+  pstm_o = pstm;
+  if (pl_last_writing == W_NUMBER_0 || pl_last_writing == W_NUMBER)
+    pl_last_writing = W_NOTHING;
+
+  Out_Char('.');
+  Out_Char('\n');
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_WRITE_A_CHAR                                                         *
  *                                                                         *
@@ -518,6 +541,11 @@ Show_Atom(int context, int atom)
 	  break;
 
 	case OTHER_ATOM:
+	  if (prop.length == 0)
+            {
+              pl_last_writing = W_NOTHING;
+              break;
+            }
 	  c = pl_atom_tbl[atom].name[prop.length - 1];
 	  c_type = pl_char_type[c];
 	  if (c_type & (UL | CL | SL | DI))
@@ -719,7 +747,6 @@ Show_List_Arg(int depth, WamWord *lst_adr)
 
 
 
-
 /*-------------------------------------------------------------------------*
  * IS_VALID_VAR_NAME                                                       *
  *                                                                         *
@@ -747,6 +774,22 @@ Is_Valid_Var_Name(char *str)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_IS_VALID_VAR_NAME_1                                                  *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Is_Valid_Var_Name_1(WamWord name_word)
+{
+  WamWord word, tag_mask;
+
+  DEREF(name_word, word, tag_mask);
+  return (tag_mask == TAG_ATM_MASK) && Is_Valid_Var_Name(pl_atom_tbl[UnTag_ATM(word)].name);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * SHOW_STRUCTURE                                                          *
  *                                                                         *
@@ -770,19 +813,22 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 
   depth--;
 
-  if (name_vars && f_n == dollar_varname_1)
+  if (name_vars && f_n == dollar_varname_1 && stc_adr >= name_number_above_H)
     {
       DEREF(Arg(stc_adr, 0), word, tag_mask);
-      p = pl_atom_tbl[UnTag_ATM(word)].name;
-      if (tag_mask == TAG_ATM_MASK && Is_Valid_Var_Name(p))
+      if (tag_mask == TAG_ATM_MASK)
 	{
-	  Out_String(p);
-	  pl_last_writing = W_IDENTIFIER;
-	  return;
+	  p = pl_atom_tbl[UnTag_ATM(word)].name;
+	  if (Is_Valid_Var_Name(p))
+	    {
+	      Out_String(p);
+	      pl_last_writing = W_IDENTIFIER;
+	      return;
+	    }
 	}
     }
 
-  if (number_vars && f_n == dollar_var_1)
+  if (number_vars && f_n == dollar_var_1 && stc_adr >= name_number_above_H)
     {
       DEREF(Arg(stc_adr, 0), word, tag_mask);
       if (tag_mask == TAG_INT_MASK && (n = UnTag_INT(word)) >= 0)
