@@ -6,7 +6,7 @@
  * Descr.: FD instruction implementation                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,13 +35,15 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_inst.c,v 1.22 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: fd_inst.c,v 1.24 2012/06/01 16:19:27 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
 
 #define FD_INST_FILE
 
+#define OBJ_INIT Fd_Inst_Initializer
+
 #include "engine_pl.h"
 #include "engine_fd.h"
 
@@ -73,7 +75,7 @@ static WamWord *TP;
 static WamWord dummy_fd_var[FD_VARIABLE_FRAME_SIZE];
 
 static PlULong always_date = -1;	/* must be always > DATE */
-static PlULong never_date = 0;	/* must be always < DATE */
+static PlULong never_date = 0;		/* must be always < DATE */
 
 void (*pl_fd_init_solver) () = Pl_Fd_Init_Solver0;	/* overwrite var of if_no_fd.c */
 void (*pl_fd_reset_solver) () = Pl_Fd_Reset_Solver0;	/* overwrite var of if_no_fd.c */
@@ -261,6 +263,22 @@ static void All_Propagations(WamWord *fdv_adr, int propag);
 
 
 
+
+/*-------------------------------------------------------------------------*
+ * FD_INST_INITIALIZER                                                     *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static void
+Fd_Inst_Initializer(void)
+{
+  pl_fd_init_solver = Pl_Fd_Init_Solver0;
+  pl_fd_reset_solver = Pl_Fd_Reset_Solver0;
+}
+
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_FD_INIT_SOLVER0                                                      *
  *                                                                         *
