@@ -6,7 +6,7 @@
  * Descr.: architecture dependent features - Header file                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arch_dep.h,v 1.23 2011/05/11 15:11:11 diaz Exp $ */
+/* $Id: arch_dep.h,v 1.25 2012/06/20 13:07:35 diaz Exp $ */
 
 #ifndef _ARCH_DEP_H
 #define _ARCH_DEP_H
@@ -101,12 +101,12 @@
 #define DIR_SEP_S                  "\\"
 #define DIR_SEP_C                  '\\'
 
-#else
+#else /* !defined(_WIN32) || defined(__CYGWIN__) */
 
 #define DIR_SEP_S                  "/"
 #define DIR_SEP_C                  '/'
 
-#endif /* defined(_WIN32) && !defined(__CYGWIN__) */
+#endif /* !defined(_WIN32) || defined(__CYGWIN__) */
 
 #if defined(M_ix86_cygwin) || defined(M_ix86_sco)
 #define Set_Line_Buf(s)            setvbuf(s, NULL, _IOLBF, 0)
@@ -144,6 +144,13 @@
 #define fgetc getc
 #endif
 
+
+#ifndef HAVE_SIGSETJMP
+#define sigjmp_buf jmp_buf
+#define sigsetjmp(jb, x) setjmp(jb)
+#define siglongjmp longjmp
+#endif
+
 				/* Fast call macros */
 #if defined(M_ix86)
 
@@ -181,7 +188,7 @@
 /* Win32 SEH macros */
 
 #if defined(_WIN32) && !defined(_WIN64) || defined(__CYGWIN__)
-#define USE_SEH 1
+#define USE_SEH
 #endif
 
 #if defined(USE_SEH)
