@@ -6,7 +6,7 @@
  * Descr.: atom table management                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom.c,v 1.20 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: atom.c,v 1.22 2012/01/10 17:46:01 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -189,17 +189,10 @@ Pl_Init_Atom(void)
   hash_weight_tbl['@'] = i;
   hash_inv_tbl[i] = '@';
 
-  for (c = 128; c < 256; c++) {
-    if (isalpha (c)) {
-      i++;
-      pl_char_type[c] = islower(c)? SL: CL;
-      hash_weight_tbl[c] = i;
-      hash_inv_tbl[i] = c;
+  for (c = 128; c < 256; c++) 
+    {
+      pl_char_type[c] = islower(c) ? SL : (isupper(c)) ? CL : EX;
     }
-    else {
-      pl_char_type[c] = EX;	/* extended char set */
-    }
-  }
 
   for (i = 0; i < 256; i++)	/* initial conv mapping = identity */
     pl_char_conv[i] = i;
