@@ -6,7 +6,7 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine1.c,v 1.28 2011/06/28 13:40:31 diaz Exp $ */
+/* $Id: engine1.c,v 1.31 2012/01/10 17:46:01 diaz Exp $ */
 
 #include "engine_pl.h"
 
@@ -116,10 +116,15 @@ Pl_Call_Compiled(CodePtr codep)
   register WamWord *rb asm("%l0") = pl_reg_bank;
   pl_ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
-#endif
+#elif defined(M_x86_64_darwin)
+
+  register WamWord *rb asm("%r12") = pl_reg_bank;
+  pl_ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
 #endif
 
+#endif /* !defined(NO_MACHINE_REG_FOR_REG_BANK) && !defined(MAP_REG_BANK) */
+
   pl_ensure_reserved = reserved_stack_space;
 #if defined(M_ix86_darwin)	/* see comment in Ma2Asm/ix86_any.c */
   asm("andl $0xfffffff0,%esp");
