@@ -6,7 +6,7 @@
  * Descr.: machine dependent features                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.c,v 1.34 2011/04/08 16:16:16 diaz Exp $ */
+/* $Id: machine.c,v 1.40 2012/06/20 13:07:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -100,7 +100,7 @@
 #define M_MAGIC1                   0x12345678
 #define M_MAGIC2                   0xdeadbeef
 
-#define UNKOWN_SYS_ERRNO           "Unknown error (%d)"
+#define UNKNOWN_SYS_ERRNO          "Unknown error (%d)"
 
 
 
@@ -139,8 +139,6 @@ static PlLong start_real_time = 0;
 
 static int cur_seed = 1;
 
-static int page_size;
-
 
 
 
@@ -148,10 +146,6 @@ static int page_size;
  * Function Prototypes             *
  *---------------------------------*/
 
-static void SIGSEGV_Handler();
-
-static char *Stack_Overflow_Err_Msg(int stk_nb);
-
 #ifdef INET_MANAGEMENT
 
 static char *Host_Name_From_Alias(struct hostent *host_entry);
@@ -189,509 +183,13 @@ Pl_Init_Machine(void)
 
 
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-
-/*-------------------------------------------------------------------------*
- * GETPAGESIZE                                                             *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-int
-getpagesize(void)
-{
-  SYSTEM_INFO si;
-
-  GetSystemInfo(&si);
-  return si.dwPageSize;
-}
-
-#endif
-
-
-
-#if HAVE_MMAP && !defined(_WIN32) && !defined(__CYGWIN__)
-#include <sys/mman.h>
-
-#if !defined(MAP_ANON) && defined(MAP_ANONYMOUS)
-#define MAP_ANON MAP_ANONYMOUS
-#endif
-#endif
-
-/*-------------------------------------------------------------------------*
- * VIRTUAL_MEM_ALLOC                                                       *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static WamWord *
-Virtual_Mem_Alloc(WamWord *addr, int length)
-{
-#if defined(_WIN32) || defined(__CYGWIN__)
-
-  addr = (WamWord *) VirtualAlloc(addr, length,
-                                  MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
-
-#elif defined(HAVE_MMAP)
-
-#ifndef MAP_ANON
-  static int fd = -1;
-
-  if (fd == -1)
-    fd = open("/dev/zero", 0);
-
-  if (fd == -1)
-    Pl_Fatal_Error(ERR_CANNOT_OPEN_DEV0, Pl_M_Sys_Err_String(errno));
-#endif /* !MAP_ANON */
-
-  addr = (WamWord *) mmap((void *) addr, length, PROT_READ | PROT_WRITE,
-                          MAP_PRIVATE
-#ifdef MMAP_NEEDS_FIXED
-                          | MAP_FIXED
-#endif
-#ifdef MAP_ANON
-                          | MAP_ANON, -1,
-#else
-                          , fd,
-#endif /* !MAP_ANON */
-                          0);
-
-  if (addr == (WamWord *) -1)
-    addr = NULL;
-
-#else
-
-  addr = (WamWord *) Calloc(length, 1);
-
-#endif
-
-  return addr;
-}
-
-
-
-#if TAG_SIZE_HIGH > 0
-/*-------------------------------------------------------------------------*
- * VIRTUAL_MEM_FREE                                                        *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static void
-Virtual_Mem_Free(WamWord *addr, int length)
-{
-#if defined(_WIN32) || defined(__CYGWIN__)
-
-  if (!VirtualFree(addr, 0, MEM_RELEASE))
-    Pl_Fatal_Error(ERR_CANNOT_FREE, GetLastError());
-
-#elif defined(HAVE_MMAP)
-
-  if (munmap((void *) addr, length) == -1)
-    Pl_Fatal_Error(ERR_CANNOT_UNMAP, Pl_M_Sys_Err_String(errno));
-
-#else
-
-  Free(addr);
-
-#endif
-}
-#endif  /* TAG_SIZE_HIGH > 0 */
-
-
-
-/*-------------------------------------------------------------------------*
- * VIRTUAL_MEM_PROTECT                                                     *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static void
-Virtual_Mem_Protect(WamWord *addr, int length)
-{
-#if defined(USE_SEH)
-  DWORD old_prot;
-
-  if (!VirtualProtect(addr, length, PAGE_NOACCESS, &old_prot))
-    Pl_Fatal_Error(ERR_CANNOT_PROTECT, GetLastError());
-
-#elif defined(HAVE_MMAP)
-
-#ifdef HAVE_MPROTECT
-  if (mprotect((void *) addr, length, PROT_NONE) == -1)
-#endif
-    if (munmap((void *) addr, length) == -1)
-      Pl_Fatal_Error(ERR_CANNOT_UNMAP, Pl_M_Sys_Err_String(errno));
-
-#else
-
-  addr[0] = M_MAGIC1;
-  addr[1] = M_MAGIC2;           /* and rest (addr[2...]) should be 0 */
-  addr[8] = 0;
-
-#endif
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_M_ALLOCATE_STACKS                                                    *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-void
-Pl_M_Allocate_Stacks(void)
-{
-  unsigned length = 0;
-  WamWord *addr;
-  int i;
-  WamWord *addr_to_try[] = {
-#ifndef MMAP_NEEDS_FIXED
-    NULL,
-#endif
-#ifdef M_MMAP_HIGH_ADR1
-    (WamWord *) M_MMAP_HIGH_ADR1,
-#endif
-#ifdef M_MMAP_HIGH_ADR2
-    (WamWord *) M_MMAP_HIGH_ADR2,
-#endif
-#ifdef M_MMAP_HIGH_ADR3
-    (WamWord *) M_MMAP_HIGH_ADR3,
-#endif
-    (WamWord *) -1 };
-
-  page_size = getpagesize() / sizeof(WamWord);
-
-  for (i = 0; i < NB_OF_STACKS; i++)
-    {
-      pl_stk_tbl[i].size = Round_Up(pl_stk_tbl[i].size, page_size);
-      length += pl_stk_tbl[i].size + page_size;
-    }
-  length *= sizeof(WamWord);
-
-  addr = NULL;
-  for(i = 0; addr == NULL && addr_to_try[i] != (WamWord *) -1; i++)
-    {
-      addr = addr_to_try[i];
-#ifdef DEBUG
-      DBGPRINTF("trying at high addr: %p --> ", addr);
-#endif
-      if (addr)
-        {
-          addr = (WamWord *) Round_Down((PlLong) addr, getpagesize());
-          addr -= length;
-        }
-#ifdef DEBUG
-      DBGPRINTF("base: %p length: %d\n", addr, length);
-#endif
-      addr = Virtual_Mem_Alloc(addr, length);
-#ifdef DEBUG
-      DBGPRINTF("obtaining: %p\n", addr);
-#endif
-#if TAG_SIZE_HIGH > 0
-      if (addr && (((PlULong) (addr) + length) >> (WORD_SIZE - TAG_SIZE_HIGH)) != 0)
-        {
-#ifdef DEBUG
-          DBGPRINTF("  -> invalid high bits addr\n");
-#endif
-          Virtual_Mem_Free(addr, length);
-          addr = NULL;
-        }
-#endif /* TAG_SIZE_HIGH > 0 */
-    }
-
-  if (addr == NULL)
-    Pl_Fatal_Error(ERR_STACKS_ALLOCATION);
-
-  for (i = 0; i < NB_OF_STACKS; i++)
-    {
-      pl_stk_tbl[i].stack = addr;
-      addr += pl_stk_tbl[i].size;
-      Virtual_Mem_Protect(addr, page_size);
-      addr += page_size;
-    }
-
-#if defined(HAVE_WORKING_SIGACTION) || \
-  defined(M_sparc_solaris) || defined(M_ix86_solaris) || \
-  defined(M_ix86_sco) || defined(M_x86_64_solaris)
-
-  {
-    struct sigaction act;
-
-    act.sa_sigaction = (void (*)()) SIGSEGV_Handler;
-    sigemptyset(&act.sa_mask);
-    act.sa_flags = SA_SIGINFO;
-
-    sigaction(SIGSEGV, &act, NULL);
-#ifdef M_darwin
-    sigaction(SIGBUS, &act, NULL);
-#endif
-  }
-
-#elif !defined(USE_SEH)
-  signal(SIGSEGV, (void (*)(int)) SIGSEGV_Handler);
-#endif
-}
-
-
-
-
-#if defined(USE_SEH)
-
-/*-------------------------------------------------------------------------*
- * WIN32_SEH_HANDLER                                                       *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-EXCEPT_DISPOSITION
-Win32_SEH_Handler(EXCEPTION_RECORD *excp_rec, void *establisher_frame,
-                  CONTEXT *context_rec, void *dispatcher_cxt)
-{
-  WamWord *addr;
-
-  if (excp_rec->ExceptionFlags)
-    return ExceptContinueSearch; /* unwind and others */
-
-  if (excp_rec->ExceptionCode != EXCEPTION_ACCESS_VIOLATION)
-    return ExceptContinueSearch;
-
-  addr = (WamWord *) excp_rec->ExceptionInformation[1];
-  SIGSEGV_Handler(addr);
-  return ExceptContinueExecution;
-}
-
-#endif
-
-
-/*-------------------------------------------------------------------------*
- * SIGSEGV_HANDLER                                                         *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-#if defined(HAVE_WORKING_SIGACTION) || \
-  defined(M_sparc_solaris) || defined(M_ix86_solaris) || defined(M_x86_64_solaris)
-void
-SIGSEGV_Handler(int sig, siginfo_t *sip)
-
-#elif defined(M_sparc_sunos)
-static void
-SIGSEGV_Handler(int sig, int code, int scp, WamWord *addr)
-
-#elif defined(M_alpha_osf)
-static void
-SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
-
-#elif defined(M_ix86_linux) || defined(M_powerpc_linux) || \
-      defined(M_alpha_linux)
-
-#ifdef LINUX_NEEDS_ASM_SIGCONTEXT // see configure.in
-#include <asm/sigcontext.h>
-#endif
-
-#if 0                           /* old linux */
-static void
-SIGSEGV_Handler(int sig, struct sigcontext_struct scp)
-#else
-static void
-SIGSEGV_Handler(int sig, struct sigcontext scp)
-#endif
-
-#elif defined(M_ix86_sco)
-#define _XOPEN_SOURCE_EXTENDED
-#include <signal.h>
-#include <sys/siginfo.h>
-static void
-SIGSEGV_Handler(int sig, siginfo_t * si)
-
-#elif defined(M_ix86_bsd) || defined(M_powerpc_bsd) || defined(M_sparc_bsd)
-static void
-SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
-
-#elif defined(_WIN32) || defined(__CYGWIN__)
-void
-SIGSEGV_Handler(WamWord *addr)
-
-#elif defined(M_mips_irix)
-#include <signal.h>
-void
-SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
-
-#elif defined(M_x86_64_linux) || defined(M_x86_64_bsd)
-void
-SIGSEGV_Handler(int sig, siginfo_t *sip, void *scp)
-
-#else
-static void
-SIGSEGV_Handler(int sig)
-#endif
-{
-#if defined(HAVE_WORKING_SIGACTION) || \
-  defined(M_sparc_solaris) || defined(M_ix86_solaris) || defined(M_x86_64_solaris)
-
-  WamWord *addr = (WamWord *) sip->si_addr;
-
-#elif defined(M_alpha_osf)
-
-  WamWord *addr = (WamWord *) (scp->sc_traparg_a0);
-
-#elif defined(M_alpha_linux)
-
-  WamWord *addr = (WamWord *) (scp.sc_fp_trigger_inst); /* why this one? */
-
-  /* WamWord *addr=(WamWord *) (scp.sc_traparg_a0); */
-
-
-#elif defined(M_ix86_linux)
-
-  WamWord *addr = (WamWord *) scp.cr2;
-
-#elif defined(M_x86_64_linux) || defined(M_x86_64_bsd)
-
-  WamWord *addr = (WamWord *) sip->si_addr;
-
-#elif defined(M_ix86_sco)
-
-  WamWord *addr = (WamWord *) si->si_addr;
-
-#elif defined(M_ix86_bsd)
-
-  WamWord *addr = (WamWord *) scp->sc_err;
-
-#elif defined(M_powerpc_linux)
-
-  WamWord *addr = (WamWord *) scp.regs->dar;
-
-#elif defined(M_mips_irix)
-
-  WamWord *addr = scp->sc_regs[16];
-
-#elif defined(USE_SEH)
-                                /* addr passed as argument */
-#else
-                                /* cannot detect fault addr */
-#ifdef __GNUC__
-#warning SIGSEGV_Handler does not know how to detect fault addr - use magic numbers
-#endif
-
-#define M_USE_MAGIC_NB_TO_DETECT_STACK_NAME
-
-#endif
-
-
-#ifdef M_USE_MAGIC_NB_TO_DETECT_STACK_NAME
-
-  M_Check_Magic_Words();
-
-#else  /* !M_USE_MAGIC_NB_TO_DETECT_STACK_NAME */
-
-  int i;
-
-#ifdef DEBUG
-  DBGPRINTF("BAD ADDRESS:%" PL_FMT_x "\n", (PlLong) addr);
-#endif
-
-  i = NB_OF_STACKS - 1;
-  if (addr < pl_stk_tbl[i].stack + pl_stk_tbl[i].size + page_size)
-    while (i >= 0)
-      {
-#ifdef DEBUG
-        DBGPRINTF("STACK[%d].stack + size: %" PL_FMT_x "\n",
-                  i, (PlLong) (pl_stk_tbl[i].stack + pl_stk_tbl[i].size));
-#endif
-        if (addr >= pl_stk_tbl[i].stack + pl_stk_tbl[i].size)
-          Pl_Fatal_Error(Stack_Overflow_Err_Msg(i));
-        i--;
-      }
-#endif /* !M_USE_MAGIC_NB_TO_DETECT_STACK_NAME */
-
-  Pl_Fatal_Error("Segmentation Violation");
-
-}
-
-
-
-
-#ifdef M_USE_MAGIC_NB_TO_DETECT_STACK_NAME
-
-#ifndef NO_USE_LINEDIT
-#include "../Linedit/linedit.h"
-#endif
-
-/*-------------------------------------------------------------------------*
- * M_CHECK_MAGIC_WORDS                                                     *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-void
-M_Check_Magic_Words(void)
-{
-  int i, err = 0;
-  WamWord *end, *top;
-  char *msg;
-  char *sever;
-
-  for (i = 0; i < NB_OF_STACKS; i++)
-    {
-      if (pl_stk_tbl[i].size == 0)
-        continue;
-
-      end = pl_stk_tbl[i].stack + pl_stk_tbl[i].size;
-      top = Stack_Top(i);
-#ifdef DEBUG
-      DBGPRINTF("stack: %s start: %p  end: %p  top: %p\n", pl_stk_tbl[i].name,
-                pl_stk_tbl[i].stack, end, top);
-#endif
-      sever = NULL;
-      if (end[0] != M_MAGIC1 || end[1] != M_MAGIC2 || end[8] != 0)
-        sever = "Probable Error";
-      else if (top < pl_stk_tbl[i].stack || top >= end)
-        sever = "Possible Error";
-
-      if (sever)
-        {
-          err++;
-          msg = Stack_Overflow_Err_Msg(i);
-#ifndef NO_USE_LINEDIT
-          if (pl_le_hook_message_box)
-            (*pl_le_hook_message_box)(sever, msg, 0);
-          else
-#endif
-            fprintf(stderr, "%s: %s\n", sever, msg);
-        }
-    }
-  if (err)
-    Pl_Exit_With_Value(1);
-}
-
-#endif
-
-
-
-/*-------------------------------------------------------------------------*
- * STACK_OVERFLOW_ERR_MSG                                                  *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static char *
-Stack_Overflow_Err_Msg(int stk_nb)
-{
-  InfStack *s = pl_stk_tbl + stk_nb;
-  char *var = s->env_var_name;
-  int size = s->size;
-  int usage = Stack_Top(stk_nb) - s->stack;
-  static char msg[256];
-
-  if (s->stack == Global_Stack)
-    size += REG_BANK_SIZE;      /* see Init_Engine */
-
-  size = Wam_Words_To_KBytes(size);
-  usage = Wam_Words_To_KBytes(usage);
-
-  if (pl_fixed_sizes || var[0] == '\0')
-    sprintf(msg, ERR_STACK_OVERFLOW_NO_ENV, s->name, size, usage);
-  else
-    sprintf(msg, ERR_STACK_OVERFLOW_ENV, s->name, size, usage, var);
-
-  return msg;
-}
-
-
-
 
 /*-------------------------------------------------------------------------*
  * PL_M_SYS_ERR_STRING                                                     *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 char *
-Pl_M_Sys_Err_String(int err_no)
+Pl_M_Sys_Err_String(int ret_val)
 {
 #ifdef M_sparc_sunos
   extern char *sys_errlist[];
@@ -699,18 +197,40 @@ Pl_M_Sys_Err_String(int err_no)
 #endif
 
   char *str;
-  static char buff[32];
+  static char buff[64];
+
+#if defined(_WIN32) || defined(__CYGWIN__)
+  if (ret_val == M_ERROR_WIN32)
+    {
+      int status =  GetLastError();
+
+      if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, status, 0, 
+			buff, sizeof(buff), NULL) == 0)
+	sprintf(buff, "Windows " UNKNOWN_SYS_ERRNO, status);
+      else
+	{			/* windows adds a ".\r\n" at end - remove it */
+	  char *p = buff + strlen(buff);
+	  while (--p > buff && (isspace(*p) || *p == '.'))
+	    {
+	    }
+	  p[1] = '\0';
+	}
+
+      return buff;
+    }
+#endif
+
 
 #if defined(M_sparc_sunos)
-  str = (err_no >= 0 && err_no < sys_nerr) ? sys_errlist[err_no] : NULL;
+  str = (errno >= 0 && errno < sys_nerr) ? sys_errlist[errno] : NULL;
 #else
-  str = strerror(err_no);
+  str = strerror(errno);
 #endif
 
   if (str)
     return str;
 
-  sprintf(buff, UNKOWN_SYS_ERRNO, err_no);
+  sprintf(buff, UNKNOWN_SYS_ERRNO, errno);
   return buff;
 }
 
@@ -1092,7 +612,7 @@ Pl_M_Absolute_Path_Name(char *src)
         )
         {
           p = dst;
-          while (isalnum(*src))
+          while (isalnum(*src) || *src == '_')
             *dst++ = *src++;
 #if defined(_WIN32) || defined(__CYGWIN__)
           if (c == '%' && *src != '%')
@@ -1120,12 +640,23 @@ Pl_M_Absolute_Path_Name(char *src)
 
   if (buff[res][0] == '~')
     {
-      if (buff[res][1] == DIR_SEP_C || buff[res][1] == '\0')    /* ~/... cf $HOME */
+      if (buff[res][1] == DIR_SEP_C || buff[res][1] == '/' ||
+	  buff[res][1] == '\0')    /* ~/... cf $HOME */
         {
-          if ((p = getenv("HOME")) == NULL && (p = getenv("HOMEPATH")) == NULL)
-            return NULL;
-
-          sprintf(buff[1 - res], "%s/%s", p, buff[res] + 1);
+	  q = NULL;;
+          if ((p = getenv("HOME")) == NULL)
+	    {
+#if defined(_WIN32) || defined(__CYGWIN__)
+	      if ((p = getenv("HOMEPATH")) == NULL)
+		return NULL;
+	      q = getenv("HOMEDRIVE");
+#else
+	      return NULL;
+#endif
+	    }
+	  if (q == NULL)
+	    q = "";
+          sprintf(buff[1 - res], "%s%s/%s", q, p, buff[res] + 1);
           res = 1 - res;
         }
 #if defined(__unix__) || defined(__CYGWIN__)
