@@ -6,7 +6,7 @@
  * Descr.: machine dependent features - Header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.h,v 1.26 2011/04/08 16:16:16 diaz Exp $ */
+/* $Id: machine.h,v 1.33 2012/06/20 13:07:35 diaz Exp $ */
 
 #ifndef _MACHINE_H
 #define _MACHINE_H
@@ -61,9 +61,7 @@
 
 void Pl_Init_Machine(void);
 
-void Pl_M_Allocate_Stacks(void);
-
-char *Pl_M_Sys_Err_String(int err_no);
+char *Pl_M_Sys_Err_String(int ret_val);
 
 PlLong Pl_M_User_Time(void);
 
@@ -134,7 +132,8 @@ void M_Check_Magic_Words(void); /* not compiled if not needed */
 
 #    define M_USED_REGS            {"15", "20", 0}
 
-#elif defined(M_x86_64) && !defined(_MSC_VER) 
+/* on M_x86_64_darwin Lion r12-r15 do not work (why ?) */
+#elif defined(M_x86_64) && !defined(_MSC_VER) && !defined(M_x86_64_darwin)
 
 #    define M_USED_REGS            {"r12", "r13", "r14", "r15", 0}
 
@@ -145,14 +144,19 @@ void M_Check_Magic_Words(void); /* not compiled if not needed */
 #endif
 
 
+
 #if defined(M_ix86) // && !defined(_WIN32) // && !defined(NO_USE_REGS)
 #define NO_MACHINE_REG_FOR_REG_BANK
 #endif
 
-
-//#if defined(_MSC_VER) && defined(M_x86_64)
-//#define NO_MACHINE_REG_FOR_REG_BANK
-//#endif
+/* In any case M_x86_64_darwin needs a reg for pl_reg_bank (default is r12)
+ * else Ma2Asm produces code ending with the following error:
+ * '32-bit absolute addressing is not supported for x86-64'
+ */
+#if defined(NO_USE_REGS) && !defined(NO_MACHINE_REG_FOR_REG_BANK) && \
+  defined(M_x86_64) && !defined(M_x86_64_darwin)
+#define NO_MACHINE_REG_FOR_REG_BANK
+#endif
 
 
 
