@@ -6,7 +6,7 @@
  * Descr.: object chaining management - header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,8 +35,9 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: obj_chain.h,v 1.20 2011/04/08 16:16:16 diaz Exp $ */
+/* $Id: obj_chain.h,v 1.23 2012/05/31 12:11:41 diaz Exp $ */
 
+#include "gp_config.h"
 #include "pl_long.h"
 
 /*---------------------------------*
@@ -98,6 +99,11 @@ static PlLong obj_chain_start = (PlLong) OBJ_CTOR;
 
 
 
-#if defined(_MSC_VER) || defined(M_darwin)
+#if (defined(_MSC_VER) || defined(M_darwin)) && !defined(OBJ_CHAIN_REVERSE_ORDER)
 #define OBJ_CHAIN_REVERSE_ORDER
 #endif
+
+#if 0
+#define OBJ_CHAIN_REVERSE_ORDER
+#endif
+
