@@ -6,7 +6,7 @@
  * Descr.: C Compiler options and WAM Configuration                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pl_config.c,v 1.27 2011/05/11 15:11:11 diaz Exp $ */
+/* $Id: pl_config.c,v 1.29 2012/06/20 13:07:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -155,8 +155,6 @@ void Generate_Tags(FILE *f, FILE *g);
 
 void Generate_Stacks(FILE *f, FILE *g);
 
-int Detect_Sigaction(void);
-
 void Pl_Fatal_Error(char *format, ...);
 
 
@@ -273,11 +271,6 @@ main(void)
 #endif
     );
 
-#if defined(__unix__) && !defined(__CYGWIN__)
-  printf("Working sigaction : %s\n", (Detect_Sigaction()) ? "Yes" : "No");
-#endif
-
-
 #ifdef COULD_COMPILE_FOR_FC
   printf("Use fast call     : %s\n",
 #ifndef NO_USE_FAST_CALL
@@ -1218,28 +1211,6 @@ Generate_Stacks(FILE *f, FILE *g)
 
 
 
-/*-------------------------------------------------------------------------*
- * DETECT_SIGACTION                                                        *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-int
-Detect_Sigaction(void)
-{
-  if (system("make try_sigaction 2>/dev/null 1>&2"))
-    return 0;
-
-  if (system("./try_sigaction") != 0)
-    return 0;
-
-
-  fprintf(fw_r, "\n#define HAVE_WORKING_SIGACTION 1\n");
-
-  return 1;
-}
-
-
-
-
 /*-------------------------------------------------------------------------*
  * PL_FATAL_ERROR                                                          *
  *                                                                         *
