@@ -6,7 +6,7 @@
  * Descr.: Detection of working sigaction                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,22 +35,23 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: try_sigaction.c,v 1.7 2011/03/29 09:26:00 diaz Exp $ */
+/* $Id: try_sigaction.c,v 1.9 2012/06/20 13:07:35 diaz Exp $ */
+
+#define _XOPEN_SOURCE_EXTENDED
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <unistd.h>
 
-#include "pl_long.h"
-
-#define BAD_ADDR  0xFEA4F
+#define BAD_ADDR  ((int *) 0xFEA4F)
 
 void
 SIGSEGV_Handler(int sig, siginfo_t * sip)
 {
-  PlLong addr = (PlLong) sip->si_addr;
+  int *addr = (int *) sip->si_addr;
 
-  exit ((addr == BAD_ADDR) ? 0 : 1);
+  _exit(addr != BAD_ADDR);
 }
 
 
@@ -63,11 +64,14 @@ main(int argc, char *argv[])
   act.sa_handler = NULL;
   act.sa_sigaction = (void (*)()) SIGSEGV_Handler;
   sigemptyset(&act.sa_mask);
-  act.sa_flags = SA_SIGINFO;
+  act.sa_flags = SA_SIGINFO | SA_RESTART;
 
   sigaction(SIGSEGV, &act, NULL);
+#if defined(SIGBUS) && SIGBUS != SIGSEGV
+  sigaction(SIGBUS, &act, NULL);
+#endif
 
-  * (PlLong *) BAD_ADDR = 128;
+  *BAD_ADDR = 128;
 
   return 1;
 }
