@@ -6,7 +6,7 @@
  * Descr.: WAM instruction implementation - header file                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_inst.h,v 1.25 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: wam_inst.h,v 1.27 2012/05/31 08:14:17 diaz Exp $ */
 
 #if 0
 #define GARBAGE_COLLECTOR
@@ -361,10 +361,12 @@ PlLong FC Pl_Switch_On_Integer(void);
 
 CodePtr FC Pl_Switch_On_Structure(SwtTbl t, int size);
 
-void FC Pl_Load_Cut_Level(WamWord *word_adr);
+WamWord FC Pl_Get_Current_Choice(void);
 
 void FC Pl_Cut(WamWord b_word);
 
+void FC Pl_Soft_Cut(WamWord b_word);
+
 void FC Pl_Global_Push_Float(double n);
 
 double FC Pl_Obtain_Float(WamWord *adr);
