@@ -6,7 +6,7 @@
  * Descr.: test file                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,17 +35,17 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: test_noecho.c,v 1.16 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: test_noecho.c,v 1.18 2012/01/10 17:46:02 diaz Exp $ */
 
 #include <stdio.h>
-#include <locale.h>
 #include <ctype.h>
 #include <stdarg.h>
 
-#include "linedit.h"
-
 #define printf Pl_LE_Printf
 
+#include "../EnginePl/gp_config.h"
+
+#include "../EnginePl/set_locale.h"
 
 #include "../W32GUICons/w32gc_interf.h"
 #ifdef GUI_CONSOLE_WITH_STACK_SIZES       /* only to test GUI Console stack size dialog box */
@@ -54,6 +54,9 @@ typedef PlLong WamWord;
 #include "../EnginePl/wam_stacks.h"
 #endif
 
+#include "linedit.h"
+
+#include <windows.h>
 
 /*---------------------------------*
  * Constants                       *
@@ -83,7 +86,10 @@ main(int argc, char *argv[])
 {
   int c;
 
-  setlocale(LC_ALL, "");
+  Set_Locale();
+  /*  SetConsoleCP(1252);
+  SetConsoleOutputCP(1252);
+  */
 
   while ((c = Pl_LE_Get_Key(0, 1)) != EOF)
     {
