@@ -6,7 +6,7 @@
  * Descr.: translation file for Linux/OSF on alpha                         *
  * Author: Alexander Diemand, Daniel Diaz                                  *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: alpha_any.c,v 1.21 2011/07/07 08:36:35 diaz Exp $ */
+/* $Id: alpha_any.c,v 1.23 2012/05/31 08:14:17 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -143,9 +143,6 @@ static char *def_inlines[] = {
                      addq    $10,0x8,$10                               \n\
                      stq     $31,2056($9)                              \n",
 
-  "Load_Cut_Level", "   # %s inlined\n\
-                     s8addq  $11,0,$1                                  \n\
-                     stq     $1,0($16)                                 \n",
 
   "Cut", "   # %s inlined\n\
                      srl     $16,3,$11                                 \n",
