@@ -6,7 +6,7 @@
  * Descr.: test file for MA translation                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: chkma.c,v 1.15 2011/06/29 07:20:18 diaz Exp $ */
+/* $Id: chkma.c,v 1.17 2012/06/20 13:07:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -1026,7 +1026,7 @@ Pl_Lookup_Pred(int func, int arity)
 }
 
 void
-Pl_M_Allocate_Stacks(void)
+Pl_Allocate_Stacks(void)
 {
 }
 AtomInf pl_atom_tbl[1];
