@@ -6,7 +6,7 @@
  * Descr.: pass 3: code generation                                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: code_gen.pl,v 1.27 2011/05/12 07:23:21 diaz Exp $ */
+/* $Id: code_gen.pl,v 1.29 2012/05/31 08:14:17 diaz Exp $ */
 
 code_generation(Head, Body, NbChunk, NbY, WamHead) :-
 	g_assign(last_pred, f),
@@ -427,16 +427,22 @@ dummy_instruction(put_value(x(X), X), f).
 :-	discontiguous(gen_inline_pred / 5).
 
 
-	% Cut inline ('$get_cut_level'/1,'$cut'/1)
+	% Cut inline ('$get_cut_level'/1, '$get_current_choice'/1, '$cut'/1, '$soft_cut'/1)
 
 gen_inline_pred('$get_cut_level', 1, [Arg], WamNext, WamArg) :-
 	cur_pred(Pred, N),
 	set_pred_info(cut, Pred, N),
 	gen_unif_arg(Arg, N, WamNext, WamArg).
 
+gen_inline_pred('$get_current_choice', 1, [var(VarName, _)], WamNext, [WamInst|WamNext]) :-
+	WamInst = get_current_choice(VarName).
+
 gen_inline_pred('$cut', 1, [var(VarName, _)], WamNext, [WamInst|WamNext]) :-
 	WamInst = cut(VarName).
 
+gen_inline_pred('$soft_cut', 1, [var(VarName, _)], WamNext, [WamInst|WamNext]) :-
+	WamInst = soft_cut(VarName).
+
 
 
 
