@@ -6,7 +6,7 @@
  * Descr.: indexing code generation                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: indexing.pl,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: indexing.pl,v 1.19 2012/05/31 08:14:18 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * Level 1:                                                                *
@@ -136,7 +136,8 @@ indexing(LCC, WamCode1) :-
 	indexing1(LCC, f, _, [_|WamCode]),       % ignore the unused label(0)
 	cur_pred(Pred, N),
 	(   test_pred_info(cut, Pred, N) ->
-	    WamCode1 = [load_cut_level(N)|WamCode]
+	    N1 is N + 1,
+	    WamCode1 = [pragma_arity(N1), get_current_choice(x(N))|WamCode]
 	;   WamCode1 = WamCode
 	),
 	allocate_labels(WamCode1, 1, _).
