@@ -6,7 +6,7 @@
  * Descr.: pass 2: internal format transformation                          *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: internal.pl,v 1.22 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: internal.pl,v 1.24 2012/05/31 08:14:18 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * predicate internal format: (I(t)=internal format of t)                  *
@@ -246,8 +246,13 @@ inline_predicate(Pred, Arity) :-
 
 
 inline_predicate('$get_cut_level', 1, _).
+
+inline_predicate('$get_current_choice', 1, _).
+
 inline_predicate('$cut', 1, _).
 
+inline_predicate('$soft_cut', 1, _).
+
 
 
 
