@@ -6,7 +6,7 @@
  * Descr.: code emission                                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_emit.pl,v 1.20 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: wam_emit.pl,v 1.25 2012/06/06 13:18:14 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * WAM Instructions                                                        *
@@ -76,8 +76,9 @@
  * retry_me_else(L)                         retry(L)                       *
  * trust_me_else_fail                       trust(L)                       *
  *                                                                         *
- * load_cut_level(A)                                                       *
+ * get_current_choice(V)                    pragma_arity(N) (for cut)      *
  * cut(V)                                                                  *
+ * soft_cut(V)                                                             *
  *                                                                         *
  * call_c(F, [T,...], [W,...])                                             *
  *   F=FctName, T=option only these options are relevant:                  *
@@ -112,7 +113,13 @@ emit_code_init(WamFile0, PlFile0) :-
 	prolog_name(Name),
 	prolog_version(Version),
 	format(Stream, '%% compiler: ~a ~a~n', [Name, Version]),
-	format(Stream, '%% file    : ~a~n', [PlFile]).
+	format(Stream, '%% file    : ~a~n', [PlFile]),
+	g_read(wam_comment, Cmt),
+	(   Cmt = '' ->
+	    true
+	;
+	    format(Stream, '%%           ~a~n', [Cmt])
+	).
 
 
 
@@ -360,6 +367,13 @@ bc_emit_lst_clause([bc(Cl, WamCode)|LCompCl], Stream) :-
 
 
 
-bc_emit_prolog_term(Stream, X) :-
-	numbervars(X, 0, _),
-	write_term(Stream, X, [numbervars(true), ignore_ops(true), quoted(true)]).
+bc_emit_prolog_term(Stream, Term) :- % create choice point for '$above'/1 write option
+	'$get_current_B'(B),
+	name_singleton_vars(Term),
+	bind_variables(Term, [exclude([Term])]),
+	write_term(Stream, Term, [numbervars(true), namevars(true), '$above'(B), ignore_ops(true), quoted(true)]),
+	fail.
+
+bc_emit_prolog_term(_, _).
+
+
