@@ -6,7 +6,7 @@
  * Descr.: compiler main (shell) program                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_comp.c,v 1.32 2011/05/11 15:11:12 diaz Exp $ */
+/* $Id: top_comp.c,v 1.36 2012/06/06 13:18:14 diaz Exp $ */
 
 #include "../EnginePl/gp_config.h"
 
@@ -649,8 +649,9 @@ Link_Cmd(void)
   static char file_out[MAXPATHLEN];
   static char buff[CMD_LINE_LENGTH];
   FileInf *f;
+#ifdef _MSC_VER
   int has_gui_console = 0;
-
+#endif
 
   if (no_fd_lib == 0 && no_fd_lib_warn)
     {
@@ -754,7 +755,9 @@ Link_Cmd(void)
     {    /* modify Linedit/Makefile.in to follow this list of ld objects */
       Find_File("w32gc_interf", OBJ_SUFFIX, buff + strlen(buff), 0);
       strcat(buff, " ");
+#ifdef _MSC_VER
       has_gui_console = 1;
+#endif
     }
 
 #ifdef _MSC_VER
@@ -849,7 +852,9 @@ Spawn_Decode_Hex(char *arg[])
   for (;;)
     {
       if (fgets(buff, sizeof(buff), f_out)) /* to avoid gcc warning warn_unused_result */
-	;
+	{
+	}
+
       if (feof(f_out))
 	break;
 
@@ -1113,11 +1118,22 @@ Parse_Arguments(int argc, char *argv[])
 	      continue;
 	    }
 
+	  if (Check_Arg(i, "--wam-comment"))
+	    {
+	      if (++i >= argc)
+		Pl_Fatal_Error("COMMENT missing after %s option", last_opt);
+
+	      Add_Last_Option(cmd_pl2wam.opt);
+	      last_opt = argv[i];
+	      Add_Last_Option(cmd_pl2wam.opt);
+	      continue;
+	    }
+
 	  if (Check_Arg(i, "--no-susp-warn") ||
 	      Check_Arg(i, "--no-singl-warn") ||
 	      Check_Arg(i, "--no-redef-error") ||
 	      Check_Arg(i, "--foreign-only") ||
-         Check_Arg(i, "--no-call-c") ||
+	      Check_Arg(i, "--no-call-c") ||
 	      Check_Arg(i, "--no-inline") ||
 	      Check_Arg(i, "--no-reorder") ||
 	      Check_Arg(i, "--no-reg-opt") ||
@@ -1429,6 +1445,7 @@ Display_Help(void)
   L(" ");
   L("Prolog to WAM compiler options:");
   L("  --pl-state FILE             read FILE to set the initial Prolog state");
+  L("  --wam-comment COMMENT       emit COMMENT as a comment in the WAM file");
   L("  --no-susp-warn              do not show warnings for suspicious predicates");
   L("  --no-singl-warn             do not show warnings for named singleton variables");
   L("  --no-redef-error            do not show errors for built-in redefinitions");
