@@ -6,7 +6,7 @@
  * Descr.: W32 GUI Console                                                 *
  * Author: Jacob Navia and Daniel Diaz                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: w32_console.c,v 1.21 2011/06/01 14:18:54 diaz Exp $ */
+/* $Id: w32_console.c,v 1.24 2012/01/10 17:46:02 diaz Exp $ */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -43,6 +43,8 @@
 #include <malloc.h>
 
 #include "../EnginePl/gp_config.h" /* only to know the value of WITH_HTMLHELP */
+#include "../EnginePl/set_locale.h"
+
 #include "w32gc_interf.h"          /* only to know Query_Stack() cmd constants */
 #include "../TopComp/prolog_path.c"
 
@@ -63,6 +65,10 @@
 #include "w32_resource.h"
 
 
+#ifdef _MSC_VER
+#define _STATIC_CPPLIB
+#endif
+
 #ifdef WITH_HTMLHELP
 
 /* HtmlHelp is used to display the doc (.chm file)
@@ -418,6 +424,12 @@ StartWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT
   MSG msg;
   HACCEL hAccelTable;
 
+  /* Reinit the locale in the DLL in case it has its own CRT
+   * (or else compile with MSVC option -MD)
+   */
+
+  Set_Locale();
+
   hInst = hInstance;
 
   event_char_in_queue = CreateEvent(NULL, FALSE, FALSE, NULL);
@@ -1215,7 +1227,9 @@ Get_Selected_File_Name(char *title, char *default_ext, char *filter)
   else
     strcpy(cwd, ".");
 
-  printf("USED DIRE: %s\n", cwd);
+#if 0
+  printf("USED DIR: %s\n", cwd);
+#endif
 
   memset(&ofn,0,sizeof(ofn));
   ofn.lStructSize = sizeof(ofn);
@@ -1624,42 +1638,41 @@ Flush_Buffer(void)
 
 
   n = wr_buffer_ptr - wr_buffer;
-  if (n == 0)
-    return;
-
-  /* the n chararacters have to be written from ec_start + posit to ec_start + posit + n - 1
-   * it is important to replacesel (not to insert) because linedit will rewrite all next
-   * chars in insert mode. Should be viewed as always in overwrite mode.
-   *
-   * Since we use the selection to replace chars, the selection should not be taken into
-   * account by other threads. This is the reason we protect the code with dont_use_selection
-   */
-
-  dont_use_selection = 1; // acquire selection
-
-  max_size = SendMessage(hwndEditControl, EM_GETLIMITTEXT, 0, 0);
-  text_size = SendMessage(hwndEditControl, WM_GETTEXTLENGTH, 0, 0);
-
-  *wr_buffer_ptr = '\0';
-
-  end = ec_start + posit + n;
-
-  // check if enough space (reserve room of "\r\n")
-  if (end >= max_size) // else delete n and write n: nothing change !
+  if (n > 0)
     {
-      int line = SendMessage(hwndEditControl, EM_LINEFROMCHAR, end - max_size, 0);
-      int line_index = SendMessage(hwndEditControl, EM_LINEINDEX, line + 1, 0);
-      SendMessage(hwndEditControl, EM_SETSEL, 0, line_index);
-      SendMessage(hwndEditControl, EM_REPLACESEL, 0, (LPARAM) wr_buffer_ptr);  /* empty string to remove lines */
+      /* the n chararacters have to be written from ec_start + posit to ec_start + posit + n - 1
+       * it is important to replacesel (not to insert) because linedit will rewrite all next
+       * chars in insert mode. Should be viewed as always in overwrite mode.
+       *
+       * Since we use the selection to replace chars, the selection should not be taken into
+       * account by other threads. This is the reason we protect the code with dont_use_selection
+       */
+
+      dont_use_selection = 1; // acquire selection
+
+      max_size = SendMessage(hwndEditControl, EM_GETLIMITTEXT, 0, 0);
       text_size = SendMessage(hwndEditControl, WM_GETTEXTLENGTH, 0, 0);
-      SendMessage(hwndEditControl, EM_SETSEL, text_size, text_size);
-      SendMessage(hwndEditControl, EM_GETSEL, (WPARAM) &ec_start, (WPARAM) &end); // re-init ec_start
+
+      *wr_buffer_ptr = '\0';
+
+      end = ec_start + posit + n;
+
+      // check if enough space (reserve room of "\r\n")
+      if (end >= max_size) // else delete n and write n: nothing change !
+	{
+	  int line = SendMessage(hwndEditControl, EM_LINEFROMCHAR, end - max_size, 0);
+	  int line_index = SendMessage(hwndEditControl, EM_LINEINDEX, line + 1, 0);
+	  SendMessage(hwndEditControl, EM_SETSEL, 0, line_index);
+	  SendMessage(hwndEditControl, EM_REPLACESEL, 0, (LPARAM) wr_buffer_ptr);  /* empty string to remove lines */
+	  text_size = SendMessage(hwndEditControl, WM_GETTEXTLENGTH, 0, 0);
+	  SendMessage(hwndEditControl, EM_SETSEL, text_size, text_size);
+	  SendMessage(hwndEditControl, EM_GETSEL, (WPARAM) &ec_start, (WPARAM) &end); // re-init ec_start
+	}
+
+      SendMessage(hwndEditControl, EM_SETSEL, ec_start + posit, ec_start + posit + n);
+      SendMessage(hwndEditControl, EM_REPLACESEL, 0, (LPARAM) wr_buffer);
+      posit += n;
     }
-
-  SendMessage(hwndEditControl, EM_SETSEL, ec_start + posit, ec_start + posit + n);
-  SendMessage(hwndEditControl, EM_REPLACESEL, 0, (LPARAM) wr_buffer);
-  posit += n;
-
   Set_Caret_Position(posit);
 
   wr_buffer_ptr = wr_buffer;
