@@ -6,7 +6,7 @@
  * Descr.: line editor <--> GUICons interface                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: w32gc_interf.c,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: w32gc_interf.c,v 1.21 2012/06/21 14:32:44 diaz Exp $ */
 
 #include <stdio.h>
 #include <windows.h>
@@ -140,7 +140,7 @@ Start_GUI(void)
   if (h == NULL)
     {
       fprintf(stderr, "warning: cannot load DLL " DLL_W32GUICONS
-	      " - text console used\n");
+	      " - text console used (error: %d)\n", (int) GetLastError());
       pl_le_hook_start = NULL;
       return;
     }
