@@ -6,7 +6,7 @@
  * Descr.: string dico management - header file                            *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bt_string.h,v 1.15 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: bt_string.h,v 1.17 2012/04/30 16:11:48 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -67,7 +67,7 @@ typedef struct
 }
 BTString;
 
-
+typedef void (*BTStrLstFct) (int no, char *str, void *info);
 
 
 /*---------------------------------*
@@ -87,4 +87,4 @@ BTNode *BT_String_Add(BTString *bt_str, char *str);
 
 BTNode *BT_String_Lookup(BTString *bt_str, char *str);
 
-void BT_String_List(BTString *bt_str, void (*fct) ());
+void BT_String_List(BTString *bt_str, BTStrLstFct fct);
