@@ -6,7 +6,7 @@
  * Descr.: code generation                                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam2ma.c,v 1.31 2011/06/10 13:58:09 diaz Exp $ */
+/* $Id: wam2ma.c,v 1.35 2012/05/31 08:14:18 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -234,13 +234,13 @@ void Emit_Obj_Initializer(void);
 
 void Emit_Exec_Directives(void);
 
-void Emit_One_Atom(int no, char *str);
+void Emit_One_Atom(int no, char *str, void *info);
 
-void Emit_One_Atom_Tagged(int no, char *str);
+void Emit_One_Atom_Tagged(int no, char *str, void *info);
 
 int Add_F_N_Tagged(char *atom, int n);
 
-void Emit_One_F_N_Tagged(int no, char *str);
+void Emit_One_F_N_Tagged(int no, char *str, void *info);
 
 void Label_Printf(char *label, ...) GCCPRINTF(1);
 
@@ -1575,15 +1575,38 @@ F_trust(ArgVal arg[])
 
 
 /*-------------------------------------------------------------------------*
- * F_LOAD_CUT_LEVEL                                                        *
+ * F_PRAGMA_ARITY                                                          *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-F_load_cut_level(ArgVal arg[])
+F_pragma_arity(ArgVal arg[])
 {
   Args1(INTEGER(a));
-  Inst_Printf("call_c", FAST "Pl_Load_Cut_Level(&X(%" PL_FMT_d "))", a);
-  cur_arity = a + 1;		/* to save X(a) in choice-points */
+
+  /* Used for for a pred/arity with cuts (not soft cuts).
+   * Since the cut level is stored in X(arity) we have to save it in choice-points
+   * This pragma adjusts the number of args to save in choice-points.
+   */
+
+  cur_arity = a;	
+
+}
+
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * F_GET_CURRENT_CHOICE                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+F_get_current_choice(ArgVal arg[])
+{
+  Args1(X_Y(xy));
+
+  Inst_Printf("call_c", FAST "Pl_Get_Current_Choice()");
+  Inst_Printf("move_ret", "%c(%" PL_FMT_d ")", c, xy);
 }
 
 
@@ -1603,6 +1626,20 @@ F_cut(ArgVal arg[])
 
 
 
+/*-------------------------------------------------------------------------*
+ * F_CUT                                                                   *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+F_soft_cut(ArgVal arg[])
+{
+  Args1(X_Y(xy));
+  Inst_Printf("call_c", FAST "Pl_Soft_Cut(%c(%" PL_FMT_d "))", c, xy);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * F_CALL_C                                                                *
  *                                                                         *
@@ -1874,7 +1911,7 @@ F_foreign_call_c(ArgVal arg[])
       n = type[i];
       c = F_Array_Letter(n);
 
-      s_dup = (mode[i] == FOREIGN_MODE_IN || FOREIGN_MODE_IN_OUT) &&
+      s_dup = (mode[i] == FOREIGN_MODE_IN || mode[i] == FOREIGN_MODE_IN_OUT) &&
 	(n == FOREIGN_TYPE_CHARS || n == FOREIGN_TYPE_CODES) &&
 	--nb_c_str != 0;
 
@@ -2162,7 +2199,7 @@ Emit_Exec_Directives(void)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Emit_One_Atom(int no, char *str)
+Emit_One_Atom(int no, char *str, void *info)
 {
   Inst_Printf("call_c", "Pl_Create_Atom(\"%s\")", str);
   Inst_Printf("move_ret", "at(%d)", no);
@@ -2176,7 +2213,7 @@ Emit_One_Atom(int no, char *str)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Emit_One_Atom_Tagged(int no, char *str)
+Emit_One_Atom_Tagged(int no, char *str, void *info)
 {
   BTNode *atom = BT_String_Lookup(&bt_atom, str);
 
@@ -2214,7 +2251,7 @@ Add_F_N_Tagged(char *atom, int n)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Emit_One_F_N_Tagged(int no, char *str)
+Emit_One_F_N_Tagged(int no, char *str, void *info)
 {
   int n;
   char *p = str + strlen(str) - 1;
