@@ -6,7 +6,7 @@
  * Descr.: parser                                                          *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_parser.c,v 1.23 2011/04/08 16:16:16 diaz Exp $ */
+/* $Id: wam_parser.c,v 1.26 2012/05/31 08:14:18 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -152,8 +152,11 @@ ParseInf inst[] = {
   {"retry", F_retry, 1, {INTEGER}},
   {"trust", F_trust, 1, {INTEGER}},
 
-  {"load_cut_level", F_load_cut_level, 1, {INTEGER}},
+  {"pragma_arity", F_pragma_arity, 1, {INTEGER}},
+
+  {"get_current_choice", F_get_current_choice, 1, {X_Y}},
   {"cut", F_cut, 1, {X_Y}},
+  {"soft_cut", F_soft_cut, 1, {X_Y}},
 
   {"call_c", F_call_c, 3, {ATOM, L1(ANY), L1(ANY)}},
 
@@ -567,7 +570,8 @@ Scanner(int complex_atom)
 	break;
 
       if (fgets(cur_line_str, sizeof(cur_line_str), file_in)) /* to avoid gcc warning warn_unused_result */
-	;
+	{
+	}
 
       if (feof(file_in))
 	return 0;
